@@ -428,13 +428,13 @@ update_contact_details (EBookBackendTpCl *tpcl, guint n_tp_contacts,
       continue;
     }
 
-    g_free (contact->name);
+    e_book_backend_tp_contact_update_name  (contact,
+        tp_contact_get_identifier (tp_contact));
+
     g_free (contact->alias);
     g_free (contact->status);
     g_free (contact->status_message);
 
-    contact->name = g_strdup (tp_contact_get_identifier (tp_contact));
-
     contact->alias = g_strdup (tp_contact_get_alias (tp_contact));
 
     contact->generic_status = presence_code_to_string (
@@ -550,6 +550,64 @@ remove_invalid_contacts (EBookBackendTpCl *tpcl, GArray *array)
   }
 }
 
+static void get_capabilities_for_members_cb (TpConnection *conn,
+    const GPtrArray *caps, const GError *error, gpointer userdata, 
+    GObject *weak_object);
+
+/* Inspect the features that are still drafts and are not yet supported by
+ * tp_connection_get_contacts_by_handle. We have to do this after we retrieved
+ * the TpContacts to avoid having the capabilities or the contact info 
+ * information before having the contact ID. */
+static void
+inspect_additional_features (EBookBackendTpCl *tpcl, GArray *contacts)
+{
+  EBookBackendTpClPrivate *priv = GET_PRIVATE (tpcl);
+  GArray *handles;
+  gint i;
+
+  handles = g_array_sized_new (TRUE, TRUE, sizeof (TpHandle), contacts->len);
+  for (i = 0; i < contacts->len; i++)
+  {
+    EBookBackendTpContact *contact;
+
+    contact = g_array_index (contacts, EBookBackendTpContact *, i);
+    g_array_append_val (handles, contact->handle);
+  }
+
+  DEBUG ("getting capabilities for all members");
+  /* FIXME: ContactCapabilities is still a draft so it's not yet supported by
+   * tp-glib. Switch to use it when it's ready */
+  tp_cli_connection_interface_capabilities_call_get_capabilities (priv->conn,
+      -1,
+      handles,
+      get_capabilities_for_members_cb,
+      NULL,
+      NULL,
+      (GObject *)tpcl);
+
+  /* FIXME: ContactInfo is still a draft. Switch to use tp-glib API when
+   * it's ready */
+  if (tp_proxy_has_interface_by_id (priv->conn, 
+        RTCOM_TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_INFO))
+  {
+    DEBUG ("getting contact info for all members");
+    rtcom_tp_cli_connection_interface_contact_info_call_get_contact_info (
+        priv->conn,
+        -1,
+        handles,
+        get_contact_info_for_members_cb,
+        NULL,
+        NULL,
+        (GObject *)tpcl);
+  }
+  else
+  {
+    DEBUG ("connection doesn't support ContactInfo interface");
+  }
+
+  g_array_free (handles, TRUE);
+}
+
 static void
 members_changed_with_details_cb (TpConnection *connection,
     guint n_tp_contacts, TpContact * const *tp_contacts, guint n_failed,
@@ -589,6 +647,9 @@ members_changed_with_details_cb (TpConnection *connection,
   remove_invalid_contacts (tpcl, closure->contacts_to_update);
 
   update_changed_members (closure, contacts);
+
+  inspect_additional_features (tpcl, contacts);
+
   g_array_free (contacts, TRUE);
 }
 
@@ -615,10 +676,6 @@ handle_members_changed_idle_cb (gpointer userdata)
   contact->flags &= \
     ~CONTACT_FLAG_FROM_ID(CONTACT_LIST_ID_GET_REMOTE_FROM_CURRENT (list_id));
 
-static void get_capabilities_for_members_cb (TpConnection *conn,
-    const GPtrArray *caps, const GError *error, gpointer userdata, 
-    GObject *weak_object);
-
 static void
 tp_channel_members_changed_cb (TpChannel *channel, const gchar *message, 
     const GArray *added, const GArray *removed, 
@@ -792,33 +849,6 @@ tp_channel_members_changed_cb (TpChannel *channel, const gchar *message,
         (TpHandle *)closure->handles_to_inspect->data,
         G_N_ELEMENTS (features), features,
         members_changed_with_details_cb, closure, NULL, G_OBJECT (tpcl));
-
-    DEBUG ("getting capabilities for added members");
-    /* FIXME: ContactCapabilities is still a draft so it's not yet supported by
-     * tp-glib. Switch to use it when it's ready */
-    tp_cli_connection_interface_capabilities_call_get_capabilities (priv->conn,
-      -1,
-      closure->handles_to_inspect,
-      get_capabilities_for_members_cb,
-      NULL,
-      NULL,
-      (GObject *)tpcl);
-    
-    /* FIXME: ContactInfo is still a draft. Switch to use tp-glib API when it's ready */
-    if (tp_proxy_has_interface_by_id (priv->conn, 
-        RTCOM_TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_INFO)) 
-    {
-      DEBUG ("getting contact info for added members");
-      rtcom_tp_cli_connection_interface_contact_info_call_get_contact_info (priv->conn,
-        -1,
-        closure->handles_to_inspect,
-        get_contact_info_for_members_cb,
-        NULL,
-        NULL,
-        (GObject *)tpcl);
-    }
-    else
-      DEBUG ("connection does not support ContactInfo interface");
   }
   else
     /* Must do this work in an idle since various signal bits are synchronous
@@ -1022,7 +1052,7 @@ aliases_changed_cb (TpConnection *conn, const GPtrArray *new_aliases,
 
 static void
 avatar_retrieved_cb (TpConnection *conn, TpHandle contact_handle,
-    const gchar *avatar_token, const GArray *new_avatar_data,
+    const gchar *new_avatar_token, const GArray *new_avatar_data,
     const gchar *new_avatar_mime, gpointer userdata,
     GObject *weak_object)
 {
@@ -1049,6 +1079,9 @@ avatar_retrieved_cb (TpConnection *conn, TpHandle contact_handle,
     g_free (contact->avatar_mime);
     contact->avatar_mime = g_strdup (new_avatar_mime);
 
+    g_free (contact->avatar_token);
+    contact->avatar_token = g_strdup (new_avatar_token);
+
     g_signal_emit (tpcl, signals[AVATAR_DATA_CHANGED], 0, contact);
   } else {
     WARNING ("got AvatarRetrieved for contact we don't know about "
@@ -1460,11 +1493,15 @@ e_book_backend_tp_cl_go_offline (EBookBackendTpCl *tpcl)
     g_free (contact->status_message);
     contact->status_message = NULL;
 
+    contact->capabilities = 0;
+
     g_array_append_val (contacts, contact);
   }
 
-  if (contacts->len > 0)
+  if (contacts->len > 0) {
     g_signal_emit (tpcl, signals[PRESENCES_CHANGED], 0, contacts);
+    g_signal_emit (tpcl, signals[CAPABILITIES_CHANGED], 0, contacts);
+  }
 
   g_hash_table_remove_all (priv->contacts_hash);
   g_array_free (contacts, TRUE);
@@ -1569,6 +1606,11 @@ update_capabilities (EBookBackendTpCl *tpcl, GArray *capabilities)
       {
         contact->capabilities |= CAP_VIDEO;
       }
+
+      if (cap->specific & TP_CHANNEL_MEDIA_CAPABILITY_IMMUTABLE_STREAMS)
+      {
+        contact->capabilities |= CAP_IMMUTABLE_STREAMS;
+      }
     }
 
     if (g_str_equal (cap->channel_type, TP_IFACE_CHANNEL_TYPE_TEXT))
@@ -1959,6 +2001,8 @@ get_contacts_cb (TpConnection *connection, guint n_tp_contacts,
     g_signal_emit (tpcl, signals[FLAGS_CHANGED], 0, contacts);
   }
 
+  inspect_additional_features (tpcl, contacts);
+
   g_free (closure);
   g_array_free (contacts, TRUE);
 
@@ -2015,34 +2059,6 @@ finish_get_channel_members (EBookBackendTpCl *tpcl, GetMembersClosure *closure)
       G_N_ELEMENTS (features), features,
       get_contacts_cb, closure, NULL, G_OBJECT (tpcl));
 
-  DEBUG ("getting capabilities for all members");
-  /* FIXME: ContactCapabilities is still a draft so it's not yet supported by
-   * tp-glib. Switch to use it when it's ready */
-  tp_cli_connection_interface_capabilities_call_get_capabilities (priv->conn,
-      -1,
-      handles,
-      get_capabilities_for_members_cb,
-      NULL,
-      NULL,
-      (GObject *)tpcl);
-
-  /* FIXME: ContactInfo is still a draft. Switch to use tp-glib API when
-   * it's ready */
-  if (tp_proxy_has_interface_by_id (priv->conn, 
-      RTCOM_TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACT_INFO)) {
-    DEBUG ("getting contact info for all members");
-    rtcom_tp_cli_connection_interface_contact_info_call_get_contact_info (
-        priv->conn,
-        -1,
-        handles,
-        get_contact_info_for_members_cb,
-        NULL,
-        NULL,
-        (GObject *)tpcl);
-  } else {
-    DEBUG ("connection doesn't support ContactInfo interface");
-  }
-
   g_array_free (handles, TRUE);
 }
 
@@ -2390,9 +2406,8 @@ e_book_backend_tp_cl_run_add_contact (EBookBackendTpCl *tpcl,
     goto out;
   }
 
-  g_free (new_contact->name);
-  new_contact->name = names[0];
-  g_free (names);
+  e_book_backend_tp_contact_update_name  (new_contact, names[0]);
+  g_strfreev (names);
 
   /* For all the lists, check if we're marked to add if so add */
   for (i = 0; i < CL_LAST_LIST; i+=3)
@@ -2644,7 +2659,7 @@ e_book_backend_tp_cl_request_avatar_data (EBookBackendTpCl *tpcl,
   guint i = 0;
   GArray *handles = NULL;
 
-  if (!contacts)
+  if (!contacts || !contacts->len)
     return TRUE;
 
   if (!verify_is_connected (tpcl, error_out))
